import type { APIRoute } from "astro";
import puppeteer from "puppeteer";
import { siteConfig } from "../../../config/data";
import * as TOML from "@iarna/toml";

interface ResumeData {
  basics: {
    name: string;
    email: string;
    website?: string;
    profiles: {
      network: string;
      username: string;
      url: string;
    }[];
  };
  summary: {
    content: string;
  };
  experience: {
    company: string;
    position: string;
    location: string;
    date: string;
    description: string[];
    url?: string;
  }[];
  education: {
    institution: string;
    degree: string;
    field: string;
    date: string;
    details?: string[];
  }[];
  skills: {
    name: string;
    level: number;
  }[];
  volunteer: {
    organization: string;
    position: string;
    date: string;
  }[];
  awards: {
    title: string;
    organization: string;
    date: string;
    description?: string;
  }[];
}

const generateResumeHTML = (data: ResumeData): string => {
  const resumeConfig = siteConfig.resume;

  const skillsHTML =
    data.skills
      ?.map((skill) => {
        const progressValue = skill.level * 20;
        return `
            <div class="mb-1">
                <div class="flex justify-between items-center mb-0.5">
                    <span class="text-xs font-medium text-gray-900">${skill.name}</span>
                    <span class="text-xs text-gray-600">${skill.level}/5</span>
                </div>
                <div class="w-full bg-gray-200 rounded-full h-2">
                    <div class="bg-blue-600 h-2 rounded-full transition-all duration-300" style="width: ${progressValue}%"></div>
                </div>
            </div>
        `;
      })
      .join("") || "";

  const experienceHTML =
    data.experience
      ?.map((exp) => {
        const descriptionList = exp.description
          .map((item) => `<li class="mb-1">${item}</li>`)
          .join("");

        return `
        <div class="mb-3 pl-2 border-l-2 border-blue-600">
            <h3 class="text-xs font-semibold text-gray-900 mb-1">${exp.position}</h3>
            <div class="text-xs text-gray-600 mb-1">
                <span class="font-medium">${exp.company}</span>
                <span class="mx-1">•</span>
                <span>${exp.date}</span>
                <span class="mx-1">•</span>
                <span>${exp.location}</span>
            </div>
            <ul class="text-xs text-gray-700 leading-tight ml-3 list-disc">
                ${descriptionList}
            </ul>
        </div>
      `;
      })
      .join("") || "";

  const educationHTML =
    data.education
      ?.map((edu) => {
        const detailsList = edu.details
          ? edu.details
              .map((detail) => `<li class="mb-1">${detail}</li>`)
              .join("")
          : "";

        return `
        <div class="mb-3 pl-2 border-l-2 border-green-600">
            <h3 class="text-xs font-semibold text-gray-900 mb-1">${edu.institution}</h3>
            <div class="text-xs text-gray-600 mb-1">
                <span class="font-medium">${edu.degree} in ${edu.field}</span>
                <span class="mx-1">•</span>
                <span>${edu.date}</span>
            </div>
            ${detailsList ? `<ul class="text-xs text-gray-700 leading-tight ml-3 list-disc">${detailsList}</ul>` : ""}
        </div>
      `;
      })
      .join("") || "";

  const volunteerHTML =
    data.volunteer
      ?.map((vol) => {
        return `
        <div class="mb-2 pl-2 border-l-2 border-purple-600">
            <h3 class="text-xs font-semibold text-gray-900 mb-1">${vol.organization}</h3>
            <div class="text-xs text-gray-600">
                <span class="font-medium">${vol.position}</span>
                <span class="mx-1">•</span>
                <span>${vol.date}</span>
            </div>
        </div>
      `;
      })
      .join("") || "";

  const awardsHTML =
    data.awards
      ?.map((award) => {
        return `
        <div class="mb-2 pl-2 border-l-2 border-yellow-600">
            <h3 class="text-xs font-semibold text-gray-900 mb-1">${award.title}</h3>
            <div class="text-xs text-gray-600 mb-1">
                <span class="font-medium">${award.organization}</span>
                <span class="mx-1">•</span>
                <span>${award.date}</span>
            </div>
            ${award.description ? `<div class="text-xs text-gray-700 leading-tight">${award.description}</div>` : ""}
        </div>
      `;
      })
      .join("") || "";

  return `
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>${data.basics.name} - Resume</title>
    <script src="https://cdn.tailwindcss.com"></script>
    <style>
        @media print {
            body {
                print-color-adjust: exact;
                -webkit-print-color-adjust: exact;
            }
        }

        .resume-container {
            max-width: 8.5in;
            min-height: 11in;
        }
    </style>
</head>
<body class="bg-white text-gray-900 text-xs leading-tight p-3">
    <div class="resume-container mx-auto">
        <header class="text-center mb-3 pb-2 border-b-2 border-gray-300">
            <h1 class="text-3xl font-bold text-gray-900 mb-1">${data.basics.name}</h1>
            <div class="flex justify-center items-center flex-wrap gap-4 text-xs text-gray-600">
                ${data.basics.email ? `<div class="flex items-center gap-1">📧 ${data.basics.email}</div>` : ""}
                ${
                  data.basics.profiles?.find((p) => p.network === "GitHub")
                    ? `<div class="flex items-center gap-1">🔗 github.com/${data.basics.profiles.find((p) => p.network === "GitHub")?.username}</div>`
                    : ""
                }
                ${
                  data.basics.profiles?.find((p) => p.network === "LinkedIn")
                    ? `<div class="flex items-center gap-1">💼 linkedin.com/in/${data.basics.profiles.find((p) => p.network === "LinkedIn")?.username}</div>`
                    : ""
                }
            </div>
        </header>

        ${
          data.summary && resumeConfig.sections.summary?.enabled
            ? `
        <section class="mb-3">
            <h2 class="text-sm font-semibold text-gray-900 mb-2 pb-1 border-b border-gray-300">
                ${resumeConfig.sections.summary.title || "Summary"}
            </h2>
            <div class="text-xs text-gray-700 leading-tight">${data.summary.content}</div>
        </section>
        `
            : ""
        }

        <div class="grid grid-cols-2 gap-4">
            <div class="space-y-4">
                ${
                  data.experience &&
                  data.experience.length > 0 &&
                  resumeConfig.sections.experience?.enabled
                    ? `
                <section>
                    <h2 class="text-sm font-semibold text-gray-900 mb-2 pb-1 border-b border-gray-300">
                        ${resumeConfig.sections.experience.title || "Experience"}
                    </h2>
                    <div class="space-y-3">
                        ${experienceHTML}
                    </div>
                </section>
                `
                    : ""
                }

                ${
                  data.volunteer &&
                  data.volunteer.length > 0 &&
                  resumeConfig.sections.volunteer?.enabled
                    ? `
                <section>
                    <h2 class="text-sm font-semibold text-gray-900 mb-2 pb-1 border-b border-gray-300">
                        ${resumeConfig.sections.volunteer.title || "Volunteer Work"}
                    </h2>
                    <div class="space-y-2">
                        ${volunteerHTML}
                    </div>
                </section>
                `
                    : ""
                }

                ${
                  data.awards &&
                  data.awards.length > 0 &&
                  resumeConfig.sections.awards?.enabled
                    ? `
                <section>
                    <h2 class="text-sm font-semibold text-gray-900 mb-2 pb-1 border-b border-gray-300">
                        ${resumeConfig.sections.awards.title || "Awards & Recognition"}
                    </h2>
                    <div class="space-y-2">
                        ${awardsHTML}
                    </div>
                </section>
                `
                    : ""
                }
            </div>

            <div class="space-y-4">
                ${
                  data.skills &&
                  data.skills.length > 0 &&
                  resumeConfig.sections.skills?.enabled
                    ? `
                <section>
                    <h2 class="text-sm font-semibold text-gray-900 mb-2 pb-1 border-b border-gray-300">
                        ${resumeConfig.sections.skills.title || "Skills"}
                    </h2>
                    <div class="space-y-1">
                        ${skillsHTML}
                    </div>
                </section>
                `
                    : ""
                }

                ${
                  data.education &&
                  data.education.length > 0 &&
                  resumeConfig.sections.education?.enabled
                    ? `
                <section>
                    <h2 class="text-sm font-semibold text-gray-900 mb-2 pb-1 border-b border-gray-300">
                        ${resumeConfig.sections.education.title || "Education"}
                    </h2>
                    <div class="space-y-3">
                        ${educationHTML}
                    </div>
                </section>
                `
                    : ""
                }
            </div>
        </div>
    </div>
</body>
</html>
    `;
};

export const GET: APIRoute = async ({ request }) => {
  try {
    if (!siteConfig.resume.tomlFile || !siteConfig.resume.tomlFile.trim()) {
      return new Response("Resume not configured", { status: 404 });
    }

    const url = new URL(request.url);
    const baseUrl = `${url.protocol}//${url.host}`;

    const response = await fetch(`${baseUrl}${siteConfig.resume.tomlFile}`);

    if (!response.ok) {
      throw new Error(
        `Failed to fetch resume: ${response.status} ${response.statusText}`,
      );
    }

    const tomlContent = await response.text();
    const resumeData: ResumeData = TOML.parse(
      tomlContent,
    ) as unknown as ResumeData;

    const htmlContent = generateResumeHTML(resumeData);

    const browser = await puppeteer.launch({
      headless: true,
      args: ["--no-sandbox", "--disable-setuid-sandbox"],
    });

    const page = await browser.newPage();

    await page.setContent(htmlContent, { waitUntil: "networkidle0" });

    const pdfBuffer = await page.pdf({
      format: "A4",
      margin: {
        top: "0.2in",
        bottom: "0.2in",
        left: "0.2in",
        right: "0.2in",
      },
      printBackground: true,
      preferCSSPageSize: false,
      scale: 0.9,
    });

    await browser.close();

    return new Response(pdfBuffer, {
      headers: {
        "Content-Type": "application/pdf",
        "Content-Disposition": `attachment; filename="Atridad_Lahiji_Resume.pdf"`,
        "Cache-Control": "public, max-age=3600",
      },
    });
  } catch (error) {
    console.error("Error generating PDF:", error);
    return new Response("Error generating PDF", { status: 500 });
  }
};
